from typing import Any

SERVER_STATUS_IN_TRANS = ...  # type: Any
SERVER_STATUS_AUTOCOMMIT = ...  # type: Any
SERVER_MORE_RESULTS_EXISTS = ...  # type: Any
SERVER_QUERY_NO_GOOD_INDEX_USED = ...  # type: Any
SERVER_QUERY_NO_INDEX_USED = ...  # type: Any
SERVER_STATUS_CURSOR_EXISTS = ...  # type: Any
SERVER_STATUS_LAST_ROW_SENT = ...  # type: Any
SERVER_STATUS_DB_DROPPED = ...  # type: Any
SERVER_STATUS_NO_BACKSLASH_ESCAPES = ...  # type: Any
SERVER_STATUS_METADATA_CHANGED = ...  # type: Any
