from typing import Any

Date = ...  # type: Any
Time = ...  # type: Any
TimeDelta = ...  # type: Any
Timestamp = ...  # type: Any

def DateFromTicks(ticks): ...
def TimeFromTicks(ticks): ...
def TimestampFromTicks(ticks): ...
